#ifndef CHUNK_H
#define CHUNK_H

#include <vector>
#include <glm/vec3.hpp>
#include "scene.h"

class Block;

class TerrainGenerator
{
public:
	virtual float func(float x, float y, float z) = 0;
	inline float func(glm::vec3 pos) {return func(pos.x, pos.y, pos.z);}
	
	virtual glm::vec3 grad(float x, float y, float z);
	inline glm::vec3 grad(glm::vec3 pos) {return grad(pos.x, pos.y, pos.z);}
};

class Chunk : public GeometryNode
{
private:
	/* -------- Variables  ------- */
	//INFO:  position of chunk in the world
	glm::vec3 m_position;
    TerrainGenerator* m_generator;
	short *m_vertexHashTable;
	std::vector<int> m_indiceList;

	static const int HASH_OFFSET[];
	static const int EDGE_TABLE[];
	static const int TRIANGLES_TABLE[][16];

	/* -------- Function ------- */
	
	/*INFO :
	 * Marching Cubes algorithm implementation from the work of :
	 * Paul Bourke (http://paulbourke.net/geometry/polygonise)
	 */
	int polygonise(glm::ivec3 pos, float* top, float* bottom);
	inline int getVertexId(int blockPos, int internalPos) {return (blockPos + HASH_OFFSET[internalPos]);}
	glm::vec3 createInterpolatedVertex(glm::vec3 p1, glm::vec3 p2, float valp1, float valp2, int hash);

public:
	Chunk(TerrainGenerator *generator);
	void generate(glm::vec3 pos);
	~Chunk();
};

#endif // CHUNK_H
