#ifndef FRAMEBUFFER_H
#define FRAMEBUFFER_H

#include "opengl.h"
#include <vector>

class Texture;

class FrameBuffer
{
protected:
    /**
     * @brief check() checks the state of the framebuffer and returns false if it isn't ready for use
     */
    bool check();

    const bool allocated;
    GLenum m_target;
    GLuint m_frameBufferId;
    std::vector<Texture*> m_textures;
    std::vector<GLuint> m_attachments;

public:
    /**
     * @brief this constructor should be used to create a framebuffer, once created,
     * use addTexture to attach textures to the framebuffer, then use initColorAttachments to
     * effectively bind them to the Framebuffer, at this point, the framebuffer should be ready to use.
     */
    FrameBuffer();

    /**
     * @brief this constructor is only used to get a handle on an existing Framebuffer created by another library
     */
    FrameBuffer(GLuint id) : allocated(false), m_target(GL_FRAMEBUFFER), m_frameBufferId(id) {}

    /**
     * @brief the destructor does not destroy the textures, however, deleteTextures will do that
     */
    ~FrameBuffer();

    void addTexture(Texture* tex, GLenum attachment);

    void addRenderBuffer(GLuint renderBufferId, GLenum attachment);

    void initColorAttachments();

    void deleteTextures();

    void setTarget(GLenum target = GL_FRAMEBUFFER) { m_target = target; }

    /**
     * @brief bindFBO binds the framebuffer
     * to unbind the framebuffer (bind the screen), you can use FrameBuffer::screen->bindFBO();
     */
    void bindFBO() const;

    /**
     * @brief getTexture returns an attached texture, indexed from their adding order.
     */
    Texture* getTexture(int texId);

    /**
     * @brief screen is a Framebuffer handle on the screen
     */
    static const FrameBuffer* screen;
};

#endif // FRAMEBUFFER_H
